@@ -13,9 +13,12 @@ try:
     from tkinter.simpledialog import askstring
 except (ImportError, ModuleNotFoundError) as error:
     print(
-        f"{error}. Please visit https://tkdocs.com/tutorial/install.html",
-        "for help installing it.",
+        f"{error}.\nPlease visit https://tkdocs.com/tutorial/install.html for"
+        " help installing it.\n\n"
+        "Or just specify an input file to by-pass the GUI.\n"
+        "Use eda-report -h for more details.",
     )
+    exit()
 
 from eda_report.document import ReportDocument
 from eda_report.exceptions import GroupbyVariableError
@@ -46,8 +49,8 @@ class EDAGUI(Frame):  # pragma: no cover
     tool* pop up to help set the report's *title*,
     *groupby variable(optional)* and *graph color*.
 
-    Afterwards, another file-dialog appears to set the desired location and
-    name for the generated report.
+    Afterwards, a final file-dialog appears to help select the destination
+    for the generated report.
 
     .. hint::
         For help with `Tk` - related issues, consider visiting `TkDocs`_.
