@@ -5,7 +5,7 @@ from typing import Optional, Sequence
 
 from pandas.core.frame import DataFrame
 
-from eda_report.univariate import analyze_univariate
+from eda_report.univariate import Variable
 from eda_report.validate import validate_multivariate_input
 
 
@@ -28,7 +28,7 @@ class MultiVariable:
 
     Example:
         .. literalinclude:: examples.txt
-           :lines: 85-113
+           :lines: 84-110
     """
 
     def __init__(self, data: Iterable) -> None:
@@ -64,7 +64,9 @@ class MultiVariable:
             str: The string representation of the ``MultiVariable`` instance.
         """
         if self.data.shape[1] == 1:
-            return str(analyze_univariate(self.data.squeeze()))
+            return str(
+                Variable(self.data.squeeze(), name=self.data.columns[0])
+            )
 
         if self.numeric_cols is None:
             numeric_info = numeric_stats = ""
@@ -182,15 +184,15 @@ class MultiVariable:
         nature = " positive" if correlation > 0 else " negative"
 
         value = abs(correlation)
-        if value >= 0.9:
+        if value >= 0.8:
             strength = "very strong"
-        elif value >= 0.7:
+        elif value >= 0.6:
             strength = "strong"
-        elif value >= 0.5:
+        elif value >= 0.4:
             strength = "moderate"
-        elif value >= 0.3:
+        elif value >= 0.2:
             strength = "weak"
-        elif value >= 0.1:
+        elif value >= 0.05:
             strength = "very weak"
         else:
             strength = "virtually no"
