@@ -1,10 +1,9 @@
 from io import BytesIO
 from multiprocessing import Pool
-from typing import Dict, Iterable, Optional, Tuple, Union
+from typing import Dict, Iterable, Optional, Sequence, Tuple, Union
 
 import matplotlib as mpl
 import numpy as np
-from cycler import cycler
 from matplotlib.colors import to_rgb
 from matplotlib.figure import Figure
 from scipy.stats import gaussian_kde, probplot
@@ -28,7 +27,7 @@ mpl.rc("boxplot", patchartist=True, vertical=False)
 mpl.rc("boxplot.medianprops", color="black")
 
 
-def savefig(figure: Figure) -> BytesIO:
+def _savefig(figure: Figure) -> BytesIO:
     """Saves the contents of a :class:`~matplotlib.figure.Figure` in PNG
     format, as bytes in a file-like object.
 
@@ -49,36 +48,24 @@ def savefig(figure: Figure) -> BytesIO:
     return graph
 
 
-def set_custom_palette(color: str, hue: Union[int, Iterable] = None) -> None:
-    """Set the default plot color-cycle to shades of the specified `color`.
+def _get_color_shades_of(color: str, num: int = None) -> Sequence:
+    """Obtain an array of `num` shades of the specified `color`.
 
     Args:
         color (str): The desired color.
-        hue (Union[int, Iterable], optional): If int, number of color shades
-            = hue. If an iterable, number of color shades = len(hue). If None,
-            number of color shades = 2. Defaults to None.
-
-    Raises:
-        TypeError: If hue is neither None nor an int nor an iterable.
+        num (int): Desired number of color shades.
     """
     color_rgb = to_rgb(color)
-    if hue is None:
-        num = 2
-    elif isinstance(hue, int):
-        num = hue
-    elif issubclass(type(hue), Iterable):
-        num = len(set(hue))
-    else:
-        raise TypeError(
-            "Invalid hue input. Expected an int or an iterable, but got "
-            f"{hue}."
-        )
-
-    color_array = np.linspace(color_rgb, (0.25, 0.25, 0.25), num=num)
-    mpl.rc("axes", prop_cycle=cycler(color=color_array))
+    return np.linspace(color_rgb, (0.25, 0.25, 0.25), num=num)
 
 
-def box_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
+def box_plot(
+    data: Iterable,
+    *,
+    label: str,
+    hue: Iterable = None,
+    color: Union[str, Sequence] = None,
+) -> Figure:
     """Get a box-plot from numeric values.
 
     Args:
@@ -86,6 +73,7 @@ def box_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
         label (str): A name for the `data`, shown in the title.
         hue (Iterable, optional): Values for grouping the `data`. Defaults to
             None.
+        color (Union[str, Sequence]): A valid matplotlib color specifier.
 
     Returns:
         matplotlib.figure.Figure: Matplotlib figure with the box-plot.
@@ -97,23 +85,39 @@ def box_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
     ax = fig.subplots()
 
     if hue is None:
-        bxplot = ax.boxplot(data, labels=[label], sym=".")
+        bxplot = ax.boxplot(
+            data,
+            labels=[label],
+            sym=".",
+            boxprops=dict(facecolor=color, alpha=0.75),
+        )
         ax.set_yticklabels("")
     else:
         hue = validate_univariate_input(hue)[original_data.notna()]
         groups = {key: series for key, series in data.groupby(hue)}
         bxplot = ax.boxplot(groups.values(), labels=groups.keys(), sym=".")
 
-    for idx, patch in enumerate(bxplot["boxes"]):
-        patch.set_facecolor(f"C{idx}")
-        patch.set_alpha(0.75)
+        if color is None:
+            colors = [f"C{idx}" for idx in range(hue.nunique())]
+        else:
+            colors = _get_color_shades_of(color, hue.nunique())
+
+        for patch, color in zip(bxplot["boxes"], reversed(colors)):
+            patch.set_facecolor(color)
+            patch.set_alpha(0.75)
 
     ax.set_title(f"Box-plot of {label}")
 
     return fig
 
 
-def kde_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
+def kde_plot(
+    data: Iterable,
+    *,
+    label: str,
+    hue: Iterable = None,
+    color: Union[str, Sequence] = None,
+) -> Figure:
     """Get a kde-plot from numeric values.
 
     Args:
@@ -121,6 +125,7 @@ def kde_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
         label (str): A name for the `data`, shown in the title.
         hue (Iterable, optional): Values for grouping the `data`. Defaults to
             None.
+        color (Union[str, Sequence]): A valid matplotlib color specifier.
 
     Returns:
         matplotlib.figure.Figure: Matplotlib figure with the kde-plot.
@@ -149,15 +154,20 @@ def kde_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
     if hue is None:
         kernel = gaussian_kde(data)
         density = kernel(eval_points)
-        ax.plot(eval_points, density, label=label)
-        ax.fill_between(eval_points, density, alpha=0.3)
+        ax.plot(eval_points, density, label=label, color=color)
+        ax.fill_between(eval_points, density, alpha=0.3, color=color)
     else:
         hue = validate_univariate_input(hue)[original_data.notna()]
-        for key, series in data.groupby(hue):
+        if color is None:
+            colors = [f"C{idx}" for idx in range(hue.nunique())]
+        else:
+            colors = _get_color_shades_of(color, hue.nunique())
+
+        for color, (key, series) in zip(colors, data.groupby(hue)):
             kernel = gaussian_kde(series)
             density = kernel(eval_points)
-            ax.plot(eval_points, density, label=key, alpha=0.75)
-            ax.fill_between(eval_points, density, alpha=0.25)
+            ax.plot(eval_points, density, label=key, alpha=0.75, color=color)
+            ax.fill_between(eval_points, density, alpha=0.25, color=color)
 
     ax.set_ylim(0)
     ax.legend()
@@ -166,14 +176,22 @@ def kde_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
     return fig
 
 
-def prob_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
+def prob_plot(
+    data: Iterable,
+    *,
+    label: str,
+    marker_color: Union[str, Sequence] = "C0",
+    line_color: Union[str, Sequence] = "#222",
+) -> Figure:
     """Get a probability-plot from numeric values.
 
     Args:
         data (Iterable): Values to plot.
         label (str): A name for the `data`, shown in the title.
-        hue (Iterable, optional): Values for grouping the `data`. Defaults to
-            None. (Present just for API consistency, for now)
+        marker_color (Union[str, Sequence]): Color for the plotted points.
+            Defaults to "C0".
+        line_color (Union[str, Sequence]): Color for the line of best fit.
+            Defaults to "#222".
 
     Returns:
         matplotlib.figure.Figure: Matplotlib figure with the probability-plot.
@@ -184,22 +202,25 @@ def prob_plot(data: Iterable, *, label: str, hue: Iterable = None) -> Figure:
     fig = Figure(figsize=(6.5, 4.5))
     ax = fig.subplots()
     probplot(data, fit=True, plot=ax)
-    ax.lines[0].set_color("C0")
-    ax.lines[1].set_color("k")
+    ax.lines[0].set_color(marker_color)
+    ax.lines[1].set_color(line_color)
     ax.set_title(f"Probability plot of {label}")
 
     return fig
 
 
-def bar_plot(data: Iterable, *, label: str) -> Figure:
+def bar_plot(
+    data: Iterable, *, label: str, color: Union[str, Sequence] = None
+) -> Figure:
     """Get a bar-plot from a sequence of values.
 
     Args:
         data (Iterable): Values to plot.
         label (str): A name for the `data`, shown in the title.
+        color (Union[str, Sequence]): A valid matplotlib color specifier.
 
     Returns:
-        matplotlib.figure.Figure: Matplotlib figure with the kde-plot.
+        matplotlib.figure.Figure: Matplotlib figure with the bar-plot.
     """
     original_data = validate_univariate_input(data)
     data = original_data.dropna()
@@ -209,7 +230,7 @@ def bar_plot(data: Iterable, *, label: str) -> Figure:
 
     # Include no more than 10 of the most common values
     top_10 = data.value_counts().nlargest(10)
-    bars = ax.bar(top_10.index, top_10, alpha=0.8)
+    bars = ax.bar(top_10.index, top_10, alpha=0.8, color=color)
     ax.bar_label(bars, labels=[f"{x:,.0f}" for x in top_10], padding=2)
 
     if (num_unique := data.nunique()) > 10:
@@ -234,33 +255,60 @@ def _plot_variable(variables_hue_and_color: Tuple) -> Tuple:
         Tuple: `variable`s name, and graphs in a dict.
     """
     variable, hue, color = variables_hue_and_color
-    set_custom_palette(color, hue)
     if variable.var_type == "numeric":
         graphs = {
-            plot_func.__name__: savefig(
-                plot_func(data=variable.data, hue=hue, label=variable.name)
-            )
-            for plot_func in [box_plot, kde_plot, prob_plot]
+            "box_plot": _savefig(
+                box_plot(
+                    data=variable.data,
+                    hue=hue,
+                    label=variable.name,
+                    color=color,
+                )
+            ),
+            "kde_plot": _savefig(
+                kde_plot(
+                    data=variable.data,
+                    hue=hue,
+                    label=variable.name,
+                    color=color,
+                )
+            ),
+            "prob_plot": _savefig(
+                prob_plot(
+                    data=variable.data, label=variable.name, marker_color=color
+                )
+            ),
         }
     else:  # {"boolean", "categorical", "datetime"}:
         graphs = {
-            "bar_plot": savefig(
-                bar_plot(data=variable.data, label=variable.name)
+            "bar_plot": _savefig(
+                bar_plot(data=variable.data, label=variable.name, color=color)
             )
         }
 
     return variable.name, graphs
 
 
-def plot_correlation(variables: Iterable) -> Figure:
-    """Create a bar chart showing the top 20 most correlated variables.
+def plot_correlation(
+    variables: Iterable,
+    max_pairs: int = 20,
+    color_pos: Union[str, Sequence] = "orangered",
+    color_neg: Union[str, Sequence] = "steelblue",
+) -> Figure:
+    """Create a bar chart showing the top ``max_pairs`` most correlated
+    variables.
 
     Args:
         variables (Iterable): 2-dimensional data.
+        max_pairs (int): The maximum number of numeric pairs to include in the
+            plot. Defaults to 20.
+        color_pos (Union[str, Sequence]): Color for positive correlation bars.
+            Defaults to "orangered".
+        color_neg (Union[str, Sequence]): Color for negative correlation bars.
+            Defaults to "steelblue".
 
     Returns:
-        matplotlib.figure.Figure: A bar-plot of correlation data in PNG format
-        as bytes.
+        matplotlib.figure.Figure: A bar-plot of correlation data.
     """
     if not isinstance(variables, MultiVariable):
         variables = MultiVariable(variables)
@@ -270,7 +318,7 @@ def plot_correlation(variables: Iterable) -> Figure:
         variables.correlation_df.unstack()  # get MultiIndex of cols
         .loc[pairs]  # select unique pairs
         .sort_values(key=abs)  # sort by magnitude
-        .tail(20)  # select top 20
+        .tail(max_pairs)  # select top `max_pairs` (default 20)
     )
     labels = corr_data.index.map(" vs ".join)
 
@@ -285,7 +333,7 @@ def plot_correlation(variables: Iterable) -> Figure:
         p.set_alpha(abs(p.get_width()))
 
         if p.get_width() < 0:
-            p.set_facecolor("steelblue")
+            p.set_facecolor(color_neg)
             ax.text(
                 p.get_x(),
                 p.get_y() + p.get_height() / 2,
@@ -295,7 +343,7 @@ def plot_correlation(variables: Iterable) -> Figure:
                 va="center",
             )
         else:
-            p.set_facecolor("orangered")
+            p.set_facecolor(color_pos)
             ax.text(
                 p.get_x(),
                 p.get_y() + p.get_height() / 2,
@@ -311,7 +359,10 @@ def plot_correlation(variables: Iterable) -> Figure:
 
 
 def regression_plot(
-    x: Iterable, y: Iterable, labels: Tuple[str, str], color: str
+    x: Iterable,
+    y: Iterable,
+    labels: Tuple[str, str],
+    color: Union[str, Sequence] = None,
 ) -> Figure:
     """Get a regression-plot from the provided pair of values.
 
@@ -319,7 +370,8 @@ def regression_plot(
         x (Iterable): Numeric values.
         y (Iterable): Numeric values.
         labels (Tuple[str, str]): Names for `x` and `y` respectively, shown in
-            axes labels.
+            axis labels.
+        color (Union[str, Sequence]): A valid matplotlib color specifier.
 
     Returns:
         matplotlib.figure.Figure: Matplotlib figure with the regression-plot.
@@ -395,10 +447,6 @@ def _plot_multivariable(
                 .tail(50)
                 .index
             )
-
-        set_custom_palette(
-            color=color,
-        )
         with Pool() as p:
             paired_data_gen = [
                 (variables.data.loc[:, pair], color) for pair in var_pairs
@@ -417,9 +465,9 @@ def _plot_multivariable(
             )
 
         return {
-            "correlation_plot": savefig(plot_correlation(variables)),
+            "correlation_plot": _savefig(plot_correlation(variables)),
             "regression_plots": {
-                var_pair: savefig(plot)
+                var_pair: _savefig(plot)
                 for var_pair, plot in bivariate_regression_plots.items()
             },
         }
